export type ICypressRuntimeConfiguration = Pick<Cypress.PluginConfigOptions, "isTextTerminal" | "testingType" | "projectRoot" | "reporter" | "specPattern" | "excludeSpecPattern" | "env">;
export type FilterSpecsMixedMode = "hide" | "show" | "empty-set";
interface IEnvironmentOverrides {
    stepDefinitions?: string | string[];
    messagesEnabled?: boolean;
    messagesOutput?: string;
    jsonEnabled?: boolean;
    jsonOutput?: string;
    htmlEnabled?: boolean;
    htmlOutput?: string;
    usageEnabled?: boolean;
    usageOutput?: string;
    prettyEnabled?: boolean;
    filterSpecsMixedMode?: FilterSpecsMixedMode;
    filterSpecs?: boolean;
    omitFiltered?: boolean;
    dryRun?: boolean;
}
export interface IBaseUserConfiguration {
    stepDefinitions?: string | string[];
    messages?: {
        enabled: boolean;
        output?: string;
    };
    json?: {
        enabled: boolean;
        output?: string;
    };
    html?: {
        enabled: boolean;
        output?: string;
    };
    usage?: {
        enabled: boolean;
        output?: string;
    };
    pretty?: {
        enabled: boolean;
    };
    filterSpecsMixedMode?: FilterSpecsMixedMode;
    filterSpecs?: boolean;
    omitFiltered?: boolean;
    dryRun?: boolean;
}
export interface IUserConfiguration extends IBaseUserConfiguration {
    e2e?: IBaseUserConfiguration;
    component?: IBaseUserConfiguration;
}
export interface IPreprocessorConfiguration {
    readonly stepDefinitions: string | string[];
    readonly messages: {
        enabled: boolean;
        output: string;
    };
    readonly json: {
        enabled: boolean;
        output: string;
    };
    readonly html: {
        enabled: boolean;
        output: string;
    };
    readonly usage: {
        enabled: boolean;
        output: string;
    };
    readonly pretty: {
        enabled: boolean;
    };
    readonly filterSpecsMixedMode: FilterSpecsMixedMode;
    readonly filterSpecs: boolean;
    readonly omitFiltered: boolean;
    readonly implicitIntegrationFolder: string;
    readonly isTrackingState: boolean;
    readonly dryRun: boolean;
}
export declare const COMPILED_REPORTER_ENTRYPOINT = "dist/subpath-entrypoints/pretty-reporter.js";
export declare function combineIntoConfiguration(configuration: IUserConfiguration, overrides: IEnvironmentOverrides, cypress: ICypressRuntimeConfiguration, implicitIntegrationFolder: string): IPreprocessorConfiguration;
export type ConfigurationFileResolver = (projectRoot: string) => unknown | Promise<unknown>;
export declare function resolve(cypressConfig: ICypressRuntimeConfiguration, environment: Record<string, unknown>, implicitIntegrationFolder: string, configurationFileResolver?: ConfigurationFileResolver): Promise<IPreprocessorConfiguration>;
export {};
